<?php

// delete /var/cache & /var/generation

namespace Novitax\Wintax\Observer;
use Magento\Framework\Event\ObserverInterface;

class NovitaxObserver implements ObserverInterface
{
  


  public function __construct( \Magento\Directory\Model\CountryFactory $countryFactory, \Magento\Framework\App\Config\ScopeConfigInterface $scope )
  {
       $this->_countryFactory = $countryFactory;
       $this->_scope = $scope;
  }

  public function execute(\Magento\Framework\Event\Observer $observer)
  {
    
     $order = $observer->getEvent()->getOrder();

     if($order->getState() == 'complete') {
               
        $order_id = $order->getId();
        $invoice_state = '';

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection();
        $tableName = $resource->getTableName('novitax');

        $sql = "SELECT status FROM $tableName WHERE id_order = $order_id";
        $result = $connection->fetchRow($sql); 
 
        if ($result && !empty($result['status'])) $invoice_state = $result['status'];

        if (in_array($invoice_state, array('new', ''))) {
          
          $xml = $this->generate_invoice($order);

          $sql = "REPLACE INTO $tableName SET status = 'new', xml = :NT_XML, id_order = $order_id";
          $binds = array('NT_XML' => $xml);
          $connection->query($sql, $binds);

          $order->addStatusHistoryComment('Számlázási adatok lementve.');
          
        

        }

     }
  
  }


  //Generate XML 
  public function generate_invoice($order) {


       
    //Build Xml

    $o = new \SimpleXMLElement('<Order></Order>');

    $o->addChild('Key', $order->getId());

    //Customer details
    $vevo = $o->addChild('Customer');
    $vevo->addChild('Id',(int)$order->getCustomerId());


    $addresses = $vevo->addChild('Addresses');

    $invoice = $addresses->addChild('Invoice');
    $invoice_address = $order->getBillingAddress()->getData();

    $invoice->addChild('Name', ($invoice_address['company']) ? htmlspecialchars($invoice_address['company'], ENT_XML1, 'UTF-8') : ($invoice_address['lastname'] .' ' . $invoice_address['firstname']));
    $invoice->addChild('ZIP', $invoice_address['postcode']);
    $invoice->addChild('City', $invoice_address['city']);
    $invoice->addChild('Street', $invoice_address['street']);
    $invoice->addChild('StreetName', $invoice_address['street']);

    $invoice->addChild('County','');

    $countryCode = $invoice_address['country_id'];
    $countryName = $this->_countryFactory->create()->loadByCode($countryCode)->getName();

    $invoice->addChild('Country', $countryName);
    $invoice->addChild('CountryCode', $countryCode);
    $invoice->addChild('TaxNumber', $invoice_address['vat_id']);



    $shipping = $addresses->addChild('Shipping');
    $shipping_address = $order->getShippingAddress()->getData();

    $shipping->addChild('Name', ($shipping_address['company']) ? htmlspecialchars($shipping_address['company'], ENT_XML1, 'UTF-8') : ($shipping_address['lastname'] .' ' . $shipping_address['firstname']));
    $shipping->addChild('ZIP', $shipping_address['postcode']);
    $shipping->addChild('City', $shipping_address['city']);
    $shipping->addChild('Street', $shipping_address['street']);
    $shipping->addChild('StreetName', $shipping_address['street']);

    $countryCode = $shipping_address['country_id'];
    $countryName = $this->_countryFactory->create()->loadByCode($countryCode)->getName();

    $shipping->addChild('County', '');
    $shipping->addChild('Country', $countryName);

    $shipping->addChild('CountryCode', $countryCode);

   
    $o->addChild('Currency', $order->getOrderCurrencyCode());


    $payment = $order->getPayment()->getMethod();
    $o->addChild('Payment', $this->get_payment_type($payment));

    $methodList     = $this->_scope->getValue('payment');



        
    $order_items = $order->getAllVisibleItems();
    $items = $o->addChild('Items');  

    foreach( $order_items as $itm ) {

      $i = $items->addChild('Item');

      $item = $itm->getData();  

      $i->addChild('Id', $item['product_id']);
      $i->addChild('Sku', $item['sku']);
      $i->addChild('Name', htmlspecialchars($item['name']));

      $i->addChild('Unit','db');        
      $i->addChild('Quantity', (int)$item['qty_ordered']);

      $i->addChild('PriceGross', $item['price_incl_tax']);

      $i->addChild('Vat', $item['tax_percent'] . '%');


    }  



    // Shipping
    if($order->getShippingInclTax() > 0) {

      $i = $items->addChild('Item');
      $i->addChild('Id', 'shipping-cost');
      $i->addChild('Sku', 'shipping-cost');

      $i->addChild('Name', 'Szállítási költség' . htmlspecialchars( $order->getTracksCollection()->getFirstItem()->getTitle() ));

      $i->addChild('Unit','db');        
      $i->addChild('Quantity',1);

      $i->addChild('PriceGross', $order->getShippingInclTax());

      $shipping_tax_percent = 100 * (($order->getShippingInclTax() / ($order->getShippingInclTax() - $order->getShippingTaxAmount())) - 1);
      $i->addChild('Vat', (int)$shipping_tax_percent . '%');

    }


    $xml = $o->asXML();
    $xml = substr($xml, strpos($xml, '?>') + 2);

    return $xml;

  }

    // fizetés módja
  public function get_payment_type($type) {

    switch ($type) {
      case 'banktransfer':  // átutalás
        return 'AT';
        break;
      case 'cashondelivery':   // utánvétel
        return 'UV';
        break;      
      default:
        return 'other - ' . $type;
        break;
    }

  }



}